package mccity.heroes.skills.enderbomb;

import com.herocraftonline.heroes.Heroes;
import com.herocraftonline.heroes.api.SkillResult;
import com.herocraftonline.heroes.characters.Hero;
import com.herocraftonline.heroes.characters.skill.ActiveSkill;
import com.herocraftonline.heroes.characters.skill.SkillConfigManager;
import com.herocraftonline.heroes.characters.skill.SkillType;
import com.herocraftonline.heroes.util.Messaging;
import com.herocraftonline.heroes.util.Setting;
import org.bukkit.Bukkit;
import org.bukkit.GameMode;
import org.bukkit.Material;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.EnderPearl;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.LivingEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.Event;
import org.bukkit.event.EventHandler;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.block.Action;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.event.entity.EntityExplodeEvent;
import org.bukkit.event.entity.ProjectileHitEvent;
import org.bukkit.event.player.PlayerInteractEvent;

import java.util.HashMap;
import java.util.Map;

public class SkillEnderBomb extends ActiveSkill {

    private final Map<EnderPearl, EnderBomb> enderBombs = new HashMap<EnderPearl, EnderBomb>();

    private boolean preventEnderpearlUse;
    private String enderpearlDenyString;

    public SkillEnderBomb(Heroes plugin) {
        super(plugin, "EnderBomb");
        setDescription("Throw enderbomb");
        setUsage("/skill enderbomb");
        setArgumentRange(0, 0);
        setIdentifiers("skill enderbomb");
        setTypes(SkillType.DAMAGING, SkillType.FIRE);

        Bukkit.getServer().getPluginManager().registerEvents(new SkillEnderBombListener(), plugin);
    }

    @Override
    public void init() {
        super.init();
        preventEnderpearlUse = SkillConfigManager.getRaw(this, "prevent-enderpearl-use", true);
        enderpearlDenyString = SkillConfigManager.getRaw(this, "enderpearl-deny-text", "Not allowed to use enderpearl");
    }

    public ConfigurationSection getDefaultConfig() {
        ConfigurationSection defaultConfig = super.getDefaultConfig();
        defaultConfig.set(Setting.REAGENT.node(), 368);
        defaultConfig.set(Setting.REAGENT_COST.node(), 1);
        defaultConfig.set(Setting.HEALTH_COST.node(), 1);
        defaultConfig.set("explosion-radius", 3.0);
        defaultConfig.set("entity-damage-multiplier", 1.5);
        defaultConfig.set("fire", false);
        defaultConfig.set("prevent-wilderness-block-damage-and-fire", true);
        defaultConfig.set("prevent-enderpearl-use", true);
        defaultConfig.set("enderpearl-deny-text", "Not allowed to use enderpearl");
        return defaultConfig;
    }

    public SkillResult use(Hero hero, String[] args) {
        float explosionRadius = (float) SkillConfigManager.getUseSetting(hero, this, "explosion-radius", 3.0, false);
        double entityDamageMultiplier = SkillConfigManager.getUseSetting(hero, this, "entity-damage-multiplier", 1.5, false);
        boolean fire = SkillConfigManager.getUseSetting(hero, this, "fire", false);
        boolean preventWildDamage = SkillConfigManager.getUseSetting(hero, this, "prevent-wilderness-block-damage-and-fire", true);

        Player player = hero.getPlayer();
        EnderPearl enderBomb = player.launchProjectile(EnderPearl.class);
        enderBombs.put(enderBomb, new EnderBomb(hero, explosionRadius, fire, preventWildDamage, entityDamageMultiplier));

        return SkillResult.NORMAL;
    }

    public String getDescription(Hero hero) {
        return getDescription();
    }

    public class SkillEnderBombListener implements Listener {

        /*
         * Ender bomb explosion detector
         * Following three events executes in the same tick for single explosion
         * This hack will possibly fails, when multiple explosions occurs at the same tick
         */
        private long enderbombHitTick = 0;

        private double currentEntityDamageMultiplier = 1.0;
        private boolean currentPreventWildDamage = false;

        @EventHandler(priority = EventPriority.LOW)
        public void onProjectileHit(ProjectileHitEvent event) {
            if (event.getEntityType() == EntityType.ENDER_PEARL) {
                EnderPearl enderPearl = (EnderPearl) event.getEntity();
                EnderBomb enderBomb = enderBombs.get(enderPearl);

                if (enderBomb != null) {
                    enderbombHitTick = getFirstWorldTime();
                    currentEntityDamageMultiplier = enderBomb.getEntityDamageMultiplier();
                    currentPreventWildDamage = enderBomb.isPreventWildDamage();

                    enderPearl.getWorld().createExplosion(enderPearl.getLocation(), enderBomb.getRadius(), enderBomb.isFire());
                    enderBombs.remove(enderPearl);
                    enderPearl.remove();
                }
            }
        }

        @EventHandler(priority = EventPriority.NORMAL, ignoreCancelled = true)
        public void onEntityDamage(EntityDamageEvent event) {
            if (getFirstWorldTime() == enderbombHitTick && event.getCause() == EntityDamageEvent.DamageCause.BLOCK_EXPLOSION) {
                if (currentEntityDamageMultiplier > 0) {
                    if (event.getEntity() instanceof LivingEntity) {
                        LivingEntity living = (LivingEntity) event.getEntity();
                        living.setNoDamageTicks(0);
                    }
                    event.setDamage((int) Math.round(event.getDamage() * currentEntityDamageMultiplier));
                } else {
                    event.setCancelled(true);
                }
            }
        }

        @EventHandler(priority = EventPriority.HIGHEST)
        public void onEntityExplode(EntityExplodeEvent event) {
            if (getFirstWorldTime() == enderbombHitTick && event.getEntity() == null) { // if enderbomb
                if (currentPreventWildDamage || event.isCancelled()) {
                    event.blockList().clear();
                    event.setYield(0.0F);
                }

                // create explosion, even if protection plugin canceled it
                event.setCancelled(false);
            }
        }

        private long getFirstWorldTime() {
            return Bukkit.getWorlds().get(0).getFullTime();
        }

        @EventHandler(priority = EventPriority.HIGH)
        public void onPlayerInteract(PlayerInteractEvent event) {
            if (!preventEnderpearlUse) return;

            Action action = event.getAction();
            if (action != Action.RIGHT_CLICK_BLOCK && action != Action.RIGHT_CLICK_AIR) return;

            Player player = event.getPlayer();
            if (player.getGameMode() == GameMode.CREATIVE) return;

            if (player.getItemInHand() != null && player.getItemInHand().getType() == Material.ENDER_PEARL) {
                event.setUseItemInHand(Event.Result.DENY);
                Messaging.send(player, enderpearlDenyString);
            }
        }
    }
}
